@@ -27,7 +27,6 @@
 #include <syslog.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include "gdm.h"
 #include "misc.h"
@@ -117,44 +116,21 @@ static void
 show_errors (GtkWidget *button, gpointer data)
 {
 	const char *file = data;
-	char *details;
 	FILE *fp;
-	struct stat s;
 	GtkWidget *sw;
 	GtkWidget *label;
 	GtkWidget *dlg = gtk_widget_get_toplevel (button);
 	GtkWidget *parent = button->parent;
 	GString *gs = g_string_new (NULL);
-	gboolean valid_utf8 = TRUE;
-
-	fp = NULL;
-	if (stat (file, &s) == 0) {
-		if (S_ISREG (s.st_mode))
-			fp = fopen (file, "r");
-		else {
-			g_string_printf (gs, "%s not a regular file!\n", file);
-		}
-	}
 
+	fp = fopen (file, "r");
 	if (fp != NULL) {
-		char buf[128];
-		int lines = 0;
-		while (fgets (buf, sizeof (buf), fp)) {
-			if ( ! g_utf8_validate (buf, -1, NULL))
-				valid_utf8 = FALSE;
+		char buf[256];
+		while (fgets (buf, sizeof (buf), fp))
 			g_string_append (gs, buf);
-			/* cap lines at 100 */
-			if (lines ++ > 100) {
-				g_string_append (gs, "\n... File too long to display ...\n");
-				break;
-			}
-		}
 		fclose (fp);
 	} else {
-		char  *loc;
-		loc = gdm_locale_to_utf8 (_("%s could not be opened"));
-		g_string_append_printf (gs, loc, file);
-		g_free (loc);
+		g_string_printf (gs, _("%s could not be opened"), file);
 	}
 
 	gtk_widget_destroy (button);
@@ -173,21 +149,15 @@ show_errors (GtkWidget *button, gpointer data)
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
 					     GTK_SHADOW_IN);
 
-	details = g_string_free (gs, FALSE);
-
-	if ( ! valid_utf8) {
-		char *tmp = gdm_locale_to_utf8 (details);
-		g_free (details);
-		details = tmp;
-	}
-
-	label = gtk_label_new (details);
+	label = gtk_label_new (gs->str);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), label);
 	gtk_widget_show (label);
 
 	gtk_box_pack_start (GTK_BOX (parent),
 			    sw, TRUE, TRUE, 0);
 
+	g_string_free (gs, TRUE);
+
 	center_window (dlg);
 }
 
@@ -311,7 +281,8 @@ gdm_error_box_full (GdmDisplay *d, GtkMessageType type, const char *error,
 	} else if (pid > 0) {
 		gdm_wait_for_extra (NULL);
 	} else {
-		gdm_error (_("gdm_error_box: Cannot fork to display error/info box"));
+		gdm_error (_("%s: Cannot fork to display error/info box"),
+			   "gdm_error_box");
 	}
 }
 
@@ -392,6 +363,7 @@ gdm_failsafe_question (GdmDisplay *d,
 						   GTK_STOCK_OK,
 						   GTK_RESPONSE_OK,
 						   NULL);
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		g_signal_connect (G_OBJECT (dlg), "delete_event",
 				  G_CALLBACK (gtk_true), NULL);
 
@@ -463,7 +435,8 @@ gdm_failsafe_question (GdmDisplay *d,
 		} 
 		close (p[0]);
 	} else {
-		gdm_error (_("gdm_failsafe_question: Cannot fork to display error/info box"));
+		gdm_error (_("%s: Cannot fork to display error/info box"),
+			   "gdm_failsafe_question");
 	}
 	return NULL;
 }
@@ -586,7 +559,8 @@ gdm_failsafe_yesno (GdmDisplay *d,
 		} 
 		close (p[0]);
 	} else {
-		gdm_error (_("gdm_failsafe_question: Cannot fork to display error/info box"));
+		gdm_error (_("%s: Cannot fork to display error/info box"),
+			   "gdm_failsafe_question");
 	}
 	return FALSE;
 }
