@@ -84,6 +84,8 @@ GdmConnection *unixconn = NULL; /* UNIX Socket connection */
 
 char *gdm_charset = NULL;
 
+int gdm_normal_runlevel = -1; /* runlevel on linux that gdm was started in */
+
 /* True if the server that was run was in actuallity not specified in the
  * config file.  That is if xdmcp was disabled and no local servers were
  * defined.  If the user kills all his local servers by mistake and keeps
@@ -100,24 +102,31 @@ int gdm_in_signal = 0;
 gchar *GdmUser = NULL;
 gchar *GdmGroup = NULL;
 gchar *GdmSessDir = NULL;
+gchar *GdmXsession = NULL;
 gchar *GdmLocaleFile = NULL;
+#if 0
+/* FIXME: Maybe just whack this */
 gchar *GdmGnomeDefaultSession = NULL;
+#endif
 gchar *GdmAutomaticLogin = NULL;
 gboolean GdmAutomaticLoginEnable = FALSE;
-gchar *GdmLocalNoPasswordUsers = NULL;
 gboolean GdmAlwaysRestartServer = FALSE;
 gchar *GdmConfigurator = NULL;
 gboolean GdmConfigAvailable = FALSE;
 gboolean GdmSystemMenu = FALSE;
+gboolean GdmChooserButton = FALSE;
 gboolean GdmBrowser = FALSE;
+gboolean GdmAddGtkModules = FALSE;
 gchar *GdmGlobalFaceDir = NULL;
 gint GdmXineramaScreen = 0;
 gchar *GdmGreeter = NULL;
 gchar *GdmRemoteGreeter = NULL;
+gchar *GdmGtkModulesList = NULL;
 gchar *GdmChooser = NULL;
 gchar *GdmLogDir = NULL;
 gchar *GdmDisplayInit = NULL;
 gchar *GdmSessionDir = NULL;
+gchar *GdmPostLogin = NULL;
 gchar *GdmPreSession = NULL;
 gchar *GdmPostSession = NULL;
 gchar *GdmFailsafeXServer = NULL;
@@ -163,6 +172,7 @@ gint  GdmFlexibleXServers = 5;
 gchar *GdmXnest = NULL;
 int GdmFirstVT = 7;
 gboolean GdmVTAllocation = TRUE;
+gboolean GdmDisallowTCP = TRUE;
 
 
 /* set in the main function */
@@ -236,16 +246,18 @@ gdm_config_parse (void)
     GdmDisplayInit = gnome_config_get_string (GDM_KEY_INITDIR);
     GdmAutomaticLoginEnable = gnome_config_get_bool (GDM_KEY_AUTOMATICLOGIN_ENABLE);
     GdmAutomaticLogin = gnome_config_get_string (GDM_KEY_AUTOMATICLOGIN);
-    GdmLocalNoPasswordUsers = gnome_config_get_string (GDM_KEY_LOCALNOPASSWORDUSERS);
     GdmAlwaysRestartServer = gnome_config_get_bool (GDM_KEY_ALWAYSRESTARTSERVER);
     GdmGreeter = gnome_config_get_string (GDM_KEY_GREETER);
     GdmRemoteGreeter = gnome_config_get_string (GDM_KEY_REMOTEGREETER);
+    GdmAddGtkModules = gnome_config_get_bool (GDM_KEY_ADD_GTK_MODULES);
+    GdmGtkModulesList = gnome_config_get_string (GDM_KEY_GTK_MODULES_LIST);	
     GdmGroup = gnome_config_get_string (GDM_KEY_GROUP);
     GdmHalt = gnome_config_get_string (GDM_KEY_HALT);
     GdmKillInitClients = gnome_config_get_bool (GDM_KEY_KILLIC);
     GdmLogDir= gnome_config_get_string (GDM_KEY_LOGDIR);
     GdmPidFile = gnome_config_get_string (GDM_KEY_PIDFILE);
     GdmSessionDir = gnome_config_get_string (GDM_KEY_SESSDIR);
+    GdmPostLogin = gnome_config_get_string (GDM_KEY_POSTLOGIN);
     GdmPreSession = gnome_config_get_string (GDM_KEY_PRESESS);
     GdmPostSession = gnome_config_get_string (GDM_KEY_POSTSESS);
     GdmFailsafeXServer = gnome_config_get_string (GDM_KEY_FAILSAFE_XSERVER);
@@ -253,6 +265,7 @@ gdm_config_parse (void)
     GdmConfigurator = gnome_config_get_string (GDM_KEY_CONFIGURATOR);
     GdmConfigAvailable = gnome_config_get_bool (GDM_KEY_CONFIG_AVAILABLE);
     GdmSystemMenu = gnome_config_get_bool (GDM_KEY_SYSMENU);
+    GdmChooserButton = gnome_config_get_bool (GDM_KEY_CHOOSER_BUTTON);
     GdmBrowser = gnome_config_get_bool (GDM_KEY_BROWSER);
     GdmGlobalFaceDir = gnome_config_get_string (GDM_KEY_FACEDIR);
     GdmXineramaScreen = gnome_config_get_int (GDM_KEY_XINERAMASCREEN);
@@ -261,9 +274,21 @@ gdm_config_parse (void)
     GdmRootPath = gnome_config_get_string (GDM_KEY_ROOTPATH);
     GdmServAuthDir = gnome_config_get_string (GDM_KEY_SERVAUTH);
     GdmSessDir = gnome_config_get_string (GDM_KEY_SESSDIR);
+    GdmXsession = gnome_config_get_string (GDM_KEY_BASEXSESSION);
+    if (ve_string_empty (GdmXsession)) {
+	    gdm_info (_("%s: BaseXsession empty, using %s/gdm/Xsession"),
+			"gdm_config_parse",
+			EXPANDED_SYSCONFDIR);
+	    g_free (GdmXsession);
+	    GdmXsession = g_strconcat (EXPANDED_SYSCONFDIR, "/gdm/Xsession",
+				       NULL);
+    }
     GdmSuspend = gnome_config_get_string (GDM_KEY_SUSPEND);
     GdmLocaleFile = gnome_config_get_string (GDM_KEY_LOCFILE);
+#if 0
+/* FIXME: Maybe just whack this */
     GdmGnomeDefaultSession = gnome_config_get_string (GDM_KEY_GNOMEDEFAULTSESSION);
+#endif
     GdmUser = gnome_config_get_string (GDM_KEY_USER);
     GdmUserAuthDir = gnome_config_get_string (GDM_KEY_UAUTHDIR);
     GdmUserAuthFile = gnome_config_get_string (GDM_KEY_UAUTHFILE);
@@ -315,6 +340,7 @@ gdm_config_parse (void)
 
     GdmFirstVT = gnome_config_get_int (GDM_KEY_FIRSTVT);    
     GdmVTAllocation = gnome_config_get_bool (GDM_KEY_VTALLOCATION);    
+    GdmDisallowTCP = gnome_config_get_bool (GDM_KEY_DISALLOWTCP);    
 
     GdmDebug = gnome_config_get_bool (GDM_KEY_DEBUG);
 
@@ -356,7 +382,7 @@ gdm_config_parse (void)
     }
 
     if (GdmTimedLoginDelay < 5) {
-	    gdm_info (_("%s: TimedLoginDelay less then 5, so I will just use 5."), "gdm_config_parse");
+	    gdm_info (_("%s: TimedLoginDelay less than 5, so I will just use 5."), "gdm_config_parse");
 	    GdmTimedLoginDelay = 5;
     }
 
@@ -407,6 +433,8 @@ gdm_config_parse (void)
 			    (GDM_KEY_SERVER_CHOOSABLE);
 		    svr->handled = gnome_config_get_bool
 			    (GDM_KEY_SERVER_HANDLED);
+		    svr->chooser = gnome_config_get_bool
+			    (GDM_KEY_SERVER_CHOOSER);
 
 		    if (ve_string_empty (svr->command)) {
 			    gdm_error (_("%s: Empty server command, "
@@ -872,8 +900,10 @@ deal_with_x_crashes (GdmDisplay *d)
 	    /* Yay we have a failsafe */
 	    if ( ! ve_string_empty (bin) &&
 		access (bin, X_OK) == 0) {
-		    gdm_info (_("deal_with_x_crashes: Trying failsafe X "
-				"server %s"), GdmFailsafeXServer);
+		    gdm_info (_("%s: Trying failsafe X "
+				"server %s"), 
+			      "deal_with_x_crashes",
+			      GdmFailsafeXServer);
 		    g_free (bin);
 		    g_free (d->command);
 		    d->command = g_strdup (GdmFailsafeXServer);
@@ -888,8 +918,9 @@ deal_with_x_crashes (GdmDisplay *d)
 	access (GdmXKeepsCrashing, X_OK|R_OK) == 0) {
 	    pid_t pid;
 
-	    gdm_info (_("deal_with_x_crashes: Running the "
-			"XKeepsCrashing script"));
+	    gdm_info (_("%s: Running the "
+			"XKeepsCrashing script"),
+		      "deal_with_x_crashes");
 
 	    pid = gdm_fork_extra ();
 
@@ -919,9 +950,11 @@ deal_with_x_crashes (GdmDisplay *d)
 		    gnome_setenv ("XLOG", xlog, TRUE);
 		    gnome_setenv ("BINDIR", EXPANDED_BINDIR, TRUE);
 		    gnome_setenv ("SBINDIR", EXPANDED_SBINDIR, TRUE);
+		    gnome_setenv ("LIBEXECDIR", EXPANDED_LIBEXECDIR, TRUE);
+		    gnome_setenv ("SYSCONFDIR", EXPANDED_SYSCONFDIR, TRUE);
 
 		    /* To enable gettext stuff in the script */
-		    gnome_setenv ("TEXTDOMAIN", PACKAGE, TRUE);
+		    gnome_setenv ("TEXTDOMAIN", GETTEXT_PACKAGE, TRUE);
 		    gnome_setenv ("TEXTDOMAINDIR", GNOMELOCALEDIR, TRUE);
 
 		    execv (argv[0], argv);
@@ -956,7 +989,7 @@ deal_with_x_crashes (GdmDisplay *d)
     /* if we have "open" we can talk to the user, not as user
      * friendly as the above script, but getting there */
     if ( ! just_abort &&
-	access (EXPANDED_SBINDIR "/gdmopen", X_OK) == 0) {
+	access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) == 0) {
 	    /* Shit if we knew what the program was to tell the user,
 	     * the above script would have been defined and we'd run
 	     * it for them */
@@ -978,6 +1011,58 @@ deal_with_x_crashes (GdmDisplay *d)
     return FALSE;
 }
 
+static void
+suspend_machine (void)
+{
+	if (GdmSuspendReal != NULL &&
+	    fork () == 0) {
+		char **argv;
+		/* In the child setup empty mask and set all signals to
+		 * default values */
+		gdm_unset_signals ();
+
+		/* Also make a new process group */
+		setsid ();
+
+		chdir ("/");
+
+		argv = ve_split (GdmSuspendReal);
+		execv (argv[0], argv);
+		/* FIXME: what about fail */
+		_exit (1);
+	}
+}
+
+#ifdef __linux__
+static void
+change_to_first_and_clear (gboolean reboot)
+{
+	/* kind of hack, we could do it ourselves if we weren't lazy */
+	if (access ("/usr/bin/chvt", X_OK) == 0)
+		system ("/usr/bin/chvt 1");
+	else if (access ("/bin/chvt", X_OK) == 0)
+		system ("/bin/chvt 1");
+	else
+		return;
+	close (1);
+	close (2);
+	open ("/dev/tty1", O_WRONLY);
+	open ("/dev/tty1", O_WRONLY);
+	/* yet again lazy */
+	if (access ("/usr/bin/clear", X_OK) == 0)
+		system ("/usr/bin/clear");
+	else if (access ("/bin/clear", X_OK) == 0)
+		system ("/bin/clear");
+	printf ("\n\n\n");
+	if (reboot)
+		printf (_("System is rebooting, please wait ..."));
+	else
+		printf (_("System is shutting down, please wait ..."));
+	printf ("\n\n\n");
+}
+#endif /* __linux__ */
+
+
 static gboolean 
 gdm_cleanup_children (void)
 {
@@ -1054,19 +1139,33 @@ gdm_cleanup_children (void)
 
     if ( ! GdmSystemMenu &&
 	(status == DISPLAY_REBOOT ||
+	 status == DISPLAY_SUSPEND ||
 	 status == DISPLAY_HALT)) {
-	    gdm_info (_("gdm_child_action: Reboot or Halt request when there is no system menu from display %s"), d->name);
+	    gdm_info (_("Reboot or Halt request when there is no system menu from display %s"), d->name);
 	    status = DISPLAY_REMANAGE;
     }
 
     if ( ! d->console &&
 	(status == DISPLAY_RESTARTGDM ||
 	 status == DISPLAY_REBOOT ||
+	 status == DISPLAY_SUSPEND ||
 	 status == DISPLAY_HALT)) {
-	    gdm_info (_("gdm_child_action: Restart, Reboot or Halt request from a non-local display %s"), d->name);
+	    gdm_info (_("Restart, Reboot or Halt request from a non-local display %s"), d->name);
 	    status = DISPLAY_REMANAGE;
     }
 
+    if (status == DISPLAY_RUN_CHOOSER) {
+	    /* use the chooser on the next run (but only if allowed) */
+	    if (GdmSystemMenu &&
+		GdmChooserButton)
+		    d->use_chooser = TRUE;
+	    status = DISPLAY_REMANAGE;
+	    /* go around the display loop detection, these are short
+	     * sessions, so this decreases the chances of the loop
+	     * detection being hit */
+	    d->last_loop_start_time = 0;
+    }
+
     if (status == DISPLAY_CHOSEN) {
 	    /* forget about this indirect id, since this
 	     * display will be dead very soon, and we don't want it
@@ -1099,7 +1198,8 @@ start_autopsy:
     switch (status) {
 	
     case DISPLAY_ABORT:		/* Bury this display for good */
-	gdm_info (_("gdm_child_action: Aborting display %s"), d->name);
+	gdm_info (_("%s: Aborting display %s"),
+		  "gdm_child_action", d->name);
 
 	if (gdm_restart_mode)
 		gdm_safe_restart ();
@@ -1111,45 +1211,46 @@ start_autopsy:
 	break;
 	
     case DISPLAY_REBOOT:	/* Reboot machine */
-	gdm_info (_("gdm_child_action: Master rebooting..."));
+	gdm_info (_("Master rebooting..."));
 
 	gdm_final_cleanup ();
 	chdir ("/");
 
+	change_to_first_and_clear (TRUE /* reboot */);
+
 	argv = ve_split (GdmRebootReal);
 	execv (argv[0], argv);
 
-	gdm_error (_("gdm_child_action: Reboot failed: %s"), strerror (errno));
+	gdm_error (_("%s: Reboot failed: %s"), 
+		   "gdm_child_action", strerror (errno));
 
 	status = DISPLAY_REMANAGE;
 	goto start_autopsy;
 	break;
 	
     case DISPLAY_HALT:		/* Halt machine */
-	gdm_info (_("gdm_child_action: Master halting..."));
+	gdm_info (_("Master halting..."));
 
 	gdm_final_cleanup ();
 	chdir ("/");
 
+	change_to_first_and_clear (FALSE /* reboot */);
+
 	argv = ve_split (GdmHaltReal);
 	execv (argv[0], argv);
 
-	gdm_error (_("gdm_child_action: Halt failed: %s"), strerror (errno));
+	gdm_error (_("%s: Halt failed: %s"),
+		   "gdm_child_action", strerror (errno));
 
 	status = DISPLAY_REMANAGE;
 	goto start_autopsy;
 	break;
 
     case DISPLAY_SUSPEND:	/* Suspend machine */
-	gdm_info (_("gdm_child_action: Master suspending..."));
-
-	gdm_final_cleanup ();
-	chdir ("/");
-
-	argv = ve_split (GdmSuspendReal);
-	execv (argv[0], argv);
-
-	gdm_error (_("gdm_child_action: Suspend failed: %s"), strerror (errno));
+	/* XXX: this is ugly, why should there be a suspend like this,
+	 * see GDM_SOP_SUSPEND_MACHINE */
+	gdm_info (_("Master suspending..."));
+	suspend_machine ();
 
 	status = DISPLAY_REMANAGE;
 	goto start_autopsy;
@@ -1175,7 +1276,7 @@ start_autopsy:
 	if (d->type == TYPE_LOCAL) {
 		time_t now = time (NULL);
 		d->x_faileds ++;
-		/* this is a much faster failing, don't even allow the 8
+		/* this is a much faster failing, don't even allow a few
 		 * seconds, just flash but for at most 30 seconds */
 		if (now - d->last_x_failed > 30) {
 			/* reset */
@@ -1201,7 +1302,7 @@ start_autopsy:
 		}
 		/* go around the display loop detection, we're doing
 		 * our own here */
-		d->last_start_time = 0;
+		d->last_loop_start_time = 0;
 	}
 	/* fall through */
 
@@ -1228,8 +1329,20 @@ start_autopsy:
 			 * start them now */
 			gdm_start_first_unborn_local (3 /* delay */);
 		}
+	} else if (d->type == TYPE_FLEXI || d->type == TYPE_FLEXI_XNEST) {
+		/* if this was a chooser session and we have chosen a host,
+		   then we don't want to unmanage, we want to manage and
+		   choose that host */
+		if (d->chosen_hostname != NULL) {
+			if ( ! gdm_display_manage (d)) {
+				gdm_display_unmanage (d);
+			}
+		} else {
+			/* else, this is a one time thing */
+			gdm_display_unmanage (d);
+		}
 	/* Remote displays will send a request to be managed */
-	} else /* TYPE_XDMCP, TYPE_FLEXI, TYPE_FLEXI_XNEST */ {
+	} else /* TYPE_XDMCP */ {
 		gdm_display_unmanage (d);
 	}
 	
@@ -1401,6 +1514,24 @@ ensure_desc_012 (void)
 	}
 }
 
+static void
+gdm_get_our_runlevel (void)
+{
+#ifdef __linux__
+	/* on linux we get our current runlevel, for use later
+	 * to detect a shutdown going on, and not mess up. */
+	if (access ("/sbin/runlevel", X_OK) == 0) {
+		char ign;
+		int rnl;
+		FILE *fp = fopen ("/sbin/runlevel", "r");
+		if (fscanf (fp, "%c %d", &ign, &rnl) == 2) {
+			gdm_normal_runlevel = rnl;
+		}
+		fclose (fp);
+	}
+#endif /* __linux__ */
+}
+
 int 
 main (int argc, char *argv[])
 {
@@ -1419,6 +1550,8 @@ main (int argc, char *argv[])
     store_argv (argc, argv);
     gdm_saveenv ();
 
+    gdm_get_our_runlevel ();
+
     bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
     textdomain (GETTEXT_PACKAGE);
 
@@ -1629,7 +1762,7 @@ write_x_servers (GdmDisplay *d)
 		char **argv;
 		char *command;
 		argv = gdm_server_resolve_command_line
-			(d, FALSE /* resolve_handled */,
+			(d, FALSE /* resolve_flags */,
 			 NULL /* vtarg */);
 		command = g_strjoinv (" ", argv);
 		g_strfreev (argv);
@@ -1698,6 +1831,32 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 	if (strncmp (msg, GDM_SOP_CHOSEN " ",
 		     strlen (GDM_SOP_CHOSEN " ")) == 0) {
 		gdm_choose_data (msg);
+	} else if (strncmp (msg, GDM_SOP_CHOSEN_LOCAL " ",
+		            strlen (GDM_SOP_CHOSEN_LOCAL " ")) == 0) {
+		GdmDisplay *d;
+		long slave_pid;
+		char *p;
+
+		if (sscanf (msg, GDM_SOP_CHOSEN_LOCAL " %ld", &slave_pid)
+		    != 1)
+			return;
+		p = strchr (msg, ' ');
+		if (p != NULL)
+			p = strchr (p+1, ' ');
+		if (p == NULL)
+			return;
+		p++;
+
+		/* Find out who this slave belongs to */
+		d = gdm_display_lookup (slave_pid);
+
+		if (d != NULL) {
+			g_free (d->chosen_hostname);
+			d->chosen_hostname = g_strdup (p);
+			gdm_debug ("Got CHOSEN_LOCAL == %s", p);
+			/* send ack */
+			send_slave_ack (d);
+		}
 	} else if (strncmp (msg, GDM_SOP_XPID " ",
 		            strlen (GDM_SOP_XPID " ")) == 0) {
 		GdmDisplay *d;
@@ -2026,6 +2185,13 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			/* send ack */
 			send_slave_ack (d);
 		}
+	} else if (strcmp (msg, GDM_SOP_SUSPEND_MACHINE) == 0) {
+		gdm_info (_("Master suspending..."));
+		if (GdmSuspendReal != NULL &&
+		    GdmSystemMenu) {
+			suspend_machine ();
+		}
+
 	}
 }
 
@@ -2227,6 +2393,7 @@ check_cookie (const char *file, const char *disp, const char *cookie)
 static void
 handle_flexi_server (GdmConnection *conn, int type, const char *server,
 		     gboolean handled,
+		     gboolean chooser,
 		     const char *xnest_disp, uid_t xnest_uid,
 		     const char *xnest_auth_file,
 		     const char *xnest_cookie)
@@ -2299,7 +2466,12 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 				      "ERROR 2 Startup errors\n");
 		return;
 	}
+
+	/* It is kind of ugly that we don't use
+	   the standard resolution for this, but
+	   oh well, this makes other things simpler */
 	display->handled = handled;
+	display->use_chooser = chooser;
 
 	if (type == TYPE_FLEXI_XNEST) {
 		GdmDisplay *parent;
@@ -2471,6 +2643,15 @@ update_config (const char *key)
 
 		notify_displays_int (GDM_NOTIFY_CONFIG_AVAILABLE, val);
 
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_CHOOSER_BUTTON)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_CHOOSER_BUTTON);
+		if (ve_bool_equal (val, GdmChooserButton))
+			goto update_config_ok;
+		GdmChooserButton = val;
+
+		notify_displays_int (GDM_NOTIFY_CHOOSER_BUTTON, val);
+
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_RETRYDELAY)) {
 		int val = gnome_config_get_int (GDM_KEY_RETRYDELAY);
@@ -2480,6 +2661,15 @@ update_config (const char *key)
 
 		notify_displays_int (GDM_NOTIFY_RETRYDELAY, val);
 
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_DISALLOWTCP)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_DISALLOWTCP);
+		if (ve_bool_equal (val, GdmDisallowTCP))
+			goto update_config_ok;
+		GdmDisallowTCP = val;
+
+		notify_displays_int (GDM_NOTIFY_DISALLOWTCP, val);
+
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_GREETER)) {
 		char *val = gnome_config_get_string (GDM_KEY_GREETER);
@@ -2639,6 +2829,7 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		}
 		handle_flexi_server (conn, TYPE_FLEXI, GdmStandardXServer,
 				     TRUE /* handled */,
+				     FALSE /* chooser */,
 				     NULL, 0, NULL, NULL);
 	} else if (strncmp (msg, GDM_SUP_FLEXI_XSERVER " ",
 		            strlen (GDM_SUP_FLEXI_XSERVER " ")) == 0) {
@@ -2680,7 +2871,12 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		}
 		g_free (name);
 
-		handle_flexi_server (conn, TYPE_FLEXI, command, svr->handled,
+		handle_flexi_server (conn, TYPE_FLEXI, command,
+				     /* It is kind of ugly that we don't use
+					the standard resolution for this, but
+					oh well, this makes other things simpler */
+				     svr->handled,
+				     svr->chooser,
 				     NULL, 0, NULL, NULL);
 	} else if (strncmp (msg, GDM_SUP_FLEXI_XNEST " ",
 		            strlen (GDM_SUP_FLEXI_XNEST " ")) == 0) {
@@ -2710,6 +2906,7 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		
 		handle_flexi_server (conn, TYPE_FLEXI_XNEST, GdmXnest,
 				     TRUE /* handled */,
+				     FALSE /* chooser */,
 				     dispname, uid, xauthfile, cookie);
 
 		g_free (dispname);
