@@ -64,6 +64,7 @@ extern gchar *GdmServAuthDir;
 extern gchar *GdmLogDir;
 extern gchar *GdmStandardXServer;
 extern gboolean GdmXdmcp;
+extern gboolean GdmDisallowTCP;
 extern gint high_display_num;
 extern pid_t extra_process;
 extern int extra_status;
@@ -133,9 +134,9 @@ gdm_server_reinit (GdmDisplay *disp)
 	gdm_sigchld_block_push ();
 
 	pid = gdm_fork_extra ();
-	if (pid < 1) {
+	if (pid < 0) {
 		/* So much work just because we can't fork */
-		if (disp->servpid > 0)
+		if (disp->servpid > 1)
 			kill (disp->servpid, SIGHUP);
 
 		gdm_sigchld_block_push ();
@@ -152,6 +153,11 @@ gdm_server_reinit (GdmDisplay *disp)
 
 		closelog ();
 
+		signal (SIGCHLD, SIG_IGN);
+		signal (SIGTERM, SIG_DFL);
+		signal (SIGINT, SIG_DFL);
+		signal (SIGHUP, SIG_DFL);
+
 		/* close things */
 		gdm_close_all_descriptors (0 /* from */, -1 /* except */);
 
@@ -166,7 +172,7 @@ gdm_server_reinit (GdmDisplay *disp)
 		dsp = XOpenDisplay (d->name);
 
 		/* Now whack the server with a SIGHUP */
-		if (disp->servpid > 0)
+		if (disp->servpid > 1)
 			kill (disp->servpid, SIGHUP);
 
 		if (dsp == NULL) {
@@ -227,7 +233,7 @@ gdm_server_stop (GdmDisplay *disp)
 	    servpid = disp->servpid;
 	    disp->servpid = 0;
 
-	    if (servpid > 0 &&
+	    if (servpid > 1 &&
 		kill (servpid, SIGTERM) == 0)
 		    ve_waitpid_no_signal (servpid, 0, 0);
 	    gdm_sigchld_block_pop ();
@@ -245,7 +251,7 @@ static gboolean
 busy_ask_user (GdmDisplay *disp)
 {
     /* if we have "open" we can talk to the user */
-    if (access (EXPANDED_SBINDIR "/gdmopen", X_OK) == 0) {
+    if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) == 0) {
 	    char *error = g_strdup_printf
 		    (_("There already appears to be an X server "
 		       "running on display %s.  Should I try another "
@@ -451,7 +457,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     if (pipe (server_signal_pipe) != 0) {
 	    gdm_error (_("%s: Error opening a pipe: %s"),
-		       "gdm_server_start", g_strerror (errno));
+		       "gdm_server_start", strerror (errno));
 	    return FALSE; 
     }
     server_signal_notified = FALSE;
@@ -463,7 +469,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     if (sigaction (SIGUSR1, &usr1, &old_usr1) < 0) {
 	    gdm_error (_("%s: Error setting up USR1 signal handler: %s"),
-		       "gdm_server_start", g_strerror (errno));
+		       "gdm_server_start", strerror (errno));
 	    close (server_signal_pipe[0]);
 	    close (server_signal_pipe[1]);
 	    return FALSE;
@@ -476,7 +482,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     if (sigaction (SIGCHLD, &chld, &old_chld) < 0) {
 	    gdm_error (_("%s: Error setting up CHLD signal handler: %s"),
-		       "gdm_server_start", g_strerror (errno));
+		       "gdm_server_start", strerror (errno));
 	    sigaction (SIGUSR1, &old_usr1, NULL);
 	    close (server_signal_pipe[0]);
 	    close (server_signal_pipe[1]);
@@ -490,7 +496,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     if (sigaction (SIGALRM, &alrm, &old_alrm) < 0) {
 	    gdm_error (_("%s: Error setting up ALRM signal handler: %s"),
-		       "gdm_server_start", g_strerror (errno));
+		       "gdm_server_start", strerror (errno));
 	    sigaction (SIGUSR1, &old_usr1, NULL);
 	    sigaction (SIGCHLD, &old_chld, NULL);
 	    close (server_signal_pipe[0]);
@@ -652,7 +658,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 	    gdm_sigchld_block_push ();
 	    pid = d->servpid;
 	    d->servpid = 0;
-	    if (pid > 0 &&
+	    if (pid > 1 &&
 		kill (pid, SIGTERM) == 0)
 		    ve_waitpid_no_signal (pid, NULL, 0);
 	    gdm_sigchld_block_pop ();
@@ -768,7 +774,7 @@ rotate_logs (const char *dname)
 
 char **
 gdm_server_resolve_command_line (GdmDisplay *disp,
-				 gboolean resolve_handled,
+				 gboolean resolve_flags,
 				 const char *vtarg)
 {
 	char *bin;
@@ -776,6 +782,7 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 	int len;
 	int i;
 	gboolean gotvtarg = FALSE;
+	gboolean query_in_arglist = FALSE;
 
 	bin = ve_first_word (disp->command);
 	if (bin == NULL) {
@@ -802,9 +809,15 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 				g_strfreev (old_argv);
 			} 
 
-			if (resolve_handled)
+			if (resolve_flags) {
 				/* Setup the handled function */
 				disp->handled = svr->handled;
+				/* never make use_chooser FALSE,
+				   it may have been set temporarily for
+				   us by the master */
+				if (svr->chooser)
+					disp->use_chooser = TRUE;
+			}
 		}
 	} else {
 		argv = ve_split (disp->command);
@@ -819,20 +832,41 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 		    (arg[3] == '\0' ||
 		     (isdigit (arg[3]) && arg[4] == '\0')))
 			gotvtarg = TRUE;
+		if (strcmp (arg, "-query") == 0 ||
+		    strcmp (arg, "-indirect") == 0)
+			query_in_arglist = TRUE;
 	}
 
-	argv = g_renew (char *, argv, len + 5);
+	argv = g_renew (char *, argv, len + 10);
 	for (i = len - 1; i >= 1; i--) {
 		argv[i+1] = argv[i];
 	}
 	/* server number is the FIRST argument, before any others */
 	argv[1] = g_strdup (disp->name);
 	len++;
+
 	if (disp->authfile != NULL) {
 		argv[len++] = g_strdup ("-auth");
 		argv[len++] = g_strdup (disp->authfile);
 	}
 
+	if (resolve_flags && disp->chosen_hostname) {
+		/* this display is NOT handled */
+		disp->handled = FALSE;
+		/* never ever ever use chooser here */
+		disp->use_chooser = FALSE;
+		/* run just one session */
+		argv[len++] = g_strdup ("-terminate");
+		argv[len++] = g_strdup ("-query");
+		argv[len++] = g_strdup (disp->chosen_hostname);
+		query_in_arglist = TRUE;
+	}
+
+	if (resolve_flags && GdmDisallowTCP && ! query_in_arglist) {
+		argv[len++] = g_strdup ("-nolisten");
+		argv[len++] = g_strdup ("tcp");
+	}
+
 	if (vtarg != NULL &&
 	    ! gotvtarg) {
 		argv[len++] = g_strdup (vtarg);
@@ -878,13 +912,14 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
     if (d->servpid > 0) {
 	    pid_t pid = d->servpid;
 	    d->servpid = 0;
-	    if (kill (pid, SIGTERM) == 0)
+	    if (pid > 1 &&
+		kill (pid, SIGTERM) == 0)
 		    ve_waitpid_no_signal (pid, NULL, 0);
     }
 
     /* Figure out the server command */
     argv = gdm_server_resolve_command_line (d,
-					    TRUE /* resolve handled */,
+					    TRUE /* resolve flags */,
 					    vtarg);
 
     command = g_strjoinv (" ", argv);
@@ -930,7 +965,8 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 		dup2 (logfd, 1);
 		dup2 (logfd, 2);
         } else {
-		gdm_error (_("gdm_server_spawn: Could not open logfile for display %s!"), d->name);
+		gdm_error (_("%s: Could not open logfile for display %s!"),
+			   "gdm_server_spawn", d->name);
 	}
 
 	
@@ -940,15 +976,18 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 	sigemptyset (&ign_signal.sa_mask);
 
 	if (sigaction (SIGUSR1, &ign_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting USR1 to SIG_IGN"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "USR1", "SIG_IGN");
 		_exit (SERVER_ABORT);
 	}
 	if (sigaction (SIGTTIN, &ign_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting TTIN to SIG_IGN"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "TTIN", "SIG_IGN");
 		_exit (SERVER_ABORT);
 	}
 	if (sigaction (SIGTTOU, &ign_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting TTOU to SIG_IGN"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "TTOU", "SIG_IGN");
 		_exit (SERVER_ABORT);
 	}
 
@@ -958,11 +997,13 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 	sigemptyset (&dfl_signal.sa_mask);
 
 	if (sigaction (SIGHUP, &dfl_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting HUP to SIG_DFL"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "HUP", "SIG_DFL");
 		_exit (SERVER_ABORT);
 	}
 	if (sigaction (SIGTERM, &dfl_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting TERM to SIG_DFL"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "TERM", "SIG_DFL");
 		_exit (SERVER_ABORT);
 	}
 
@@ -1054,14 +1095,16 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 
 	execv (argv[0], argv);
 	
-	gdm_error (_("gdm_server_spawn: Xserver not found: %s"), command);
+	gdm_error (_("%s: Xserver not found: %s"), 
+		   "gdm_server_spawn", command);
 	
 	_exit (SERVER_ABORT);
 	
     case -1:
 	g_strfreev (argv);
 	g_free (command);
-	gdm_error (_("gdm_server_spawn: Can't fork Xserver process!"));
+	gdm_error (_("%s: Can't fork Xserver process!"),
+		   "gdm_server_spawn");
 	d->servpid = 0;
 	d->servstat = SERVER_DEAD;
 
@@ -1070,7 +1113,8 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
     default:
 	g_strfreev (argv);
 	g_free (command);
-	gdm_debug ("gdm_server_spawn: Forked server on pid %d", (int)pid);
+	gdm_debug ("%s: Forked server on pid %d", 
+		   "gdm_server_spawn", (int)pid);
 	break;
     }
 }
@@ -1196,6 +1240,7 @@ gdm_server_alloc (gint id, const gchar *command)
 
     d->x_servers_order = -1;
 
+    d->last_loop_start_time = 0;
     d->last_start_time = 0;
     d->retry_count = 0;
     d->disabled = FALSE;
