@@ -43,6 +43,35 @@ extern gboolean GdmAllowRoot;
 extern gboolean GdmAllowRemoteRoot;
 extern gint GdmRetryDelay;
 
+static void
+print_cant_auth_errbox (void)
+{
+	gboolean is_capslock = FALSE;
+	const char *basemsg;
+	char *msg;
+	char *ret;
+
+	ret = gdm_slave_greeter_ctl (GDM_QUERY_CAPSLOCK, "");
+	if ( ! ve_string_empty (ret))
+		is_capslock = TRUE;
+	g_free (ret);
+
+	basemsg = _("\nIncorrect username or password.  "
+		    "Letters must be typed in the correct "
+		    "case.");
+	if (is_capslock) {
+		msg = g_strconcat (basemsg, "  ",
+				   _("Please make sure the "
+				     "Caps Lock key is not "
+				     "enabled."),
+				   NULL);
+	} else {
+		msg = g_strdup (basemsg);
+	}
+	gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, msg);
+	g_free (msg);
+}
+
 /**
  * gdm_verify_user:
  * @username: Name of user or NULL if we should ask
@@ -58,7 +87,7 @@ extern gint GdmRetryDelay;
 gchar *
 gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboolean local) 
 {
-    gchar *login, *passwd, *ppasswd, *auth_errmsg;
+    gchar *login, *passwd, *ppasswd;
     struct passwd *pwent;
     struct spwd *sp;
 
@@ -68,7 +97,7 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
     if (username == NULL) {
 	    /* Ask for the user's login */
 	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, _("Please enter your username"));
-	    login = gdm_slave_greeter_ctl (GDM_LOGIN, _("Username:"));
+	    login = gdm_slave_greeter_ctl (GDM_PROMPT, _("Username:"));
 	    if (login == NULL ||
 		gdm_slave_greeter_check_interruption ()) {
 		    if (local)
@@ -80,6 +109,7 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
     } else {
 	    login = g_strdup (username);
     }
+    gdm_slave_greeter_ctl_no_ret (GDM_SETLOGIN, login);
 
     pwent = getpwnam (login);
 
@@ -88,21 +118,29 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
     /* Lookup shadow password */
     sp = getspnam (login);
     
-    if (sp)
-	ppasswd = g_strdup (sp->sp_pwdp);
-    else
-	ppasswd = NULL;
+    /* Use shadow password when available */
+    if (sp != NULL) {
+	    ppasswd = g_strdup (sp->sp_pwdp);
+    } else {
+	    /* In case shadow password cannot be retrieved (when using NIS
+	       authentication for example), use standard passwd */
+	    if (pwent != NULL &&
+		pwent->pw_passwd != NULL)
+		    ppasswd = g_strdup (pwent->pw_passwd);
+	    else
+		    /* If no password can be retrieved, set it to NULL */
+		    ppasswd = NULL;
+    }
     
     endspent ();
 
     /* Request the user's password */
     if (pwent != NULL &&
-        (ve_string_empty (ppasswd) ||
-	 (local && gdm_is_a_no_password_user (login)))) {
+        ve_string_empty (ppasswd)) {
 	    /* eeek a passwordless account */
 	    passwd = g_strdup ("");
     } else {
-	    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
+	    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password:"));
 	    if (passwd == NULL)
 		    passwd = g_strdup ("");
 	    if (gdm_slave_greeter_check_interruption ()) {
@@ -120,17 +158,10 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
 
     if (pwent == NULL) {
 	    sleep (GdmRetryDelay);
-	    gdm_error (_("Couldn't authenticate user"));
-	    /* FIXME: Hmm, how are we sure that the login is username
-	     * and password.  That is the most common case but not
-	     * necessarily true, this message needs to be changed
-	     * to allow for such cases */
-	    auth_errmsg = g_strdup_printf
-		    (_("\nIncorrect username or password. "
-		       "Letters must be typed in the correct case. "  
-		       "Please make sure the Caps Lock key is not enabled."));
-	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
-	    g_free (auth_errmsg);
+	    gdm_error (_("Couldn't authenticate user \"%s\""), login);
+
+	    print_cant_auth_errbox ();
+
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -141,16 +172,10 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
     if (ppasswd == NULL || (ppasswd[0] != '\0' &&
 			    strcmp (crypt (passwd, ppasswd), ppasswd) != 0)) {
 	    sleep (GdmRetryDelay);
-	    /* FIXME: Hmm, how are we sure that the login is username
-	     * and password.  That is the most common case but not
-	     * necessarily true, this message needs to be changed
-	     * to allow for such cases */
-	    auth_errmsg = g_strdup_printf
-		    (_("\nIncorrect username or password. "
-		       "Letters must be typed in the correct case. "  
-		       "Please make sure the Caps Lock key is not enabled."));
-	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
-	    g_free (auth_errmsg);
+	    gdm_error (_("Couldn't authenticate user \"%s\""), login);
+
+	    print_cant_auth_errbox ();
+
 	    g_free (login);
 	    g_free (passwd);
 	    g_free (ppasswd);
@@ -216,10 +241,13 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
  */
 
 gboolean
-gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display) 
+gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
+		       char **new_login) 
 {
 	struct passwd *pwent;
 
+	*new_login = NULL;
+
 	pwent = getpwnam (login);
 	if (pwent == NULL) {
 		gdm_error (_("Cannot get passwd structure for %s"), login);
