@@ -32,6 +32,59 @@
 
 int response = -999;
 
+static gboolean
+gdm_check (void)
+{
+	GtkWidget *dialog;
+	FILE *fp = NULL;
+	long pid;
+	char *pidfile;
+
+	pidfile = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+					GDM_KEY_PIDFILE);
+
+	pid = 0;
+	if (pidfile != NULL)
+		fp = fopen (pidfile, "r");
+	if (fp != NULL) {
+		fscanf (fp, "%ld", &pid);
+		fclose (fp);
+	}
+
+	g_free (pidfile);
+
+	errno = 0;
+	if (pid <= 1 ||
+	    (kill (pid, 0) < 0 &&
+	     errno != EPERM)) {
+		dialog = gtk_message_dialog_new
+			(NULL /* parent */,
+			 GTK_DIALOG_MODAL /* flags */,
+			 GTK_MESSAGE_WARNING,
+			 GTK_BUTTONS_OK,
+			 "foo");
+		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+		gtk_label_set_markup
+			 (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
+			  _("<b>GDM (The GNOME Display Manager) "
+			    "is not running.</b>\n\n"
+			    "You might in fact be using a different "
+			    "display manager, such as KDM "
+			    "(KDE Display Manager or xdm).\n"
+			    "If you still wish to use this feature, "
+			    "either start GDM your self or ask your "
+			    "system administrator to start GDM."));
+		gtk_widget_show_all (dialog);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+
 static void
 dialog_response (GtkWidget *dialog, int res, gpointer data)
 {
@@ -45,6 +98,8 @@ main (int argc, char *argv[])
 	GtkWidget *dialog;
 	GtkWidget *photo;
 	gboolean face_browser;
+	char *greeter;
+	char *remotegreeter;
 	int max_size;
 	char *last_pix;
 
@@ -62,9 +117,21 @@ main (int argc, char *argv[])
 	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
 	face_browser = gnome_config_get_bool (GDM_KEY_BROWSER);
 	max_size = gnome_config_get_int (GDM_KEY_MAXFILE);
+	greeter = gnome_config_get_string (GDM_KEY_GREETER);
+	remotegreeter = gnome_config_get_string (GDM_KEY_REMOTEGREETER);
 	gnome_config_pop_prefix ();
 
-	if ( ! face_browser) {
+	if ( ! gdm_check ()) {
+		return 1;
+	}
+
+	/* HACK */
+	/* only warn if gdmlogin is set for both local and remote greeter,
+	 * the graphical greeter does a different setup thingie for
+	 * the face browser and it would be hard to figure out here ... */
+	if ( ! face_browser &&
+	     strstr (greeter, "gdmlogin") != NULL &&
+	     strstr (remotegreeter, "gdmlogin") != NULL) {
 		GtkWidget *d;
 		d = gtk_message_dialog_new (NULL /* parent */,
 					    GTK_DIALOG_MODAL /* flags */,
@@ -75,6 +142,7 @@ main (int argc, char *argv[])
 					      "system administrator to enable "
 					      "it\nin the GDM configurator "
 					      "program."));
+		gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
 	}
@@ -87,6 +155,7 @@ main (int argc, char *argv[])
 					      GTK_STOCK_OK,
 					      GTK_RESPONSE_OK,
 					      NULL);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    gtk_label_new (_("Select a photograph to show "
 					     "in the facebrowser:")),
@@ -127,6 +196,7 @@ main (int argc, char *argv[])
 						    GTK_MESSAGE_WARNING,
 						    GTK_BUTTONS_OK,
 						    _("No picture selected."));
+			gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 			gtk_dialog_run (GTK_DIALOG (d));
 			gtk_widget_destroy (d);
 		} else if (s.st_size > max_size) {
@@ -138,16 +208,17 @@ main (int argc, char *argv[])
 						    _("The picture is too large and "
 						      "the system administrator\n"
 						      "disallowed pictures larger "
-						      "then %d bytes to\n"
+						      "than %d bytes to\n"
 						      "show in the face browser"),
 						    max_size);
+			gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 			gtk_dialog_run (GTK_DIALOG (d));
 			gtk_widget_destroy (d);
 		} else {
 			char buf[4096];
 			size_t size;
 			char *photofile = g_strconcat (g_get_home_dir (),
-						       "/.gnome2/photo",
+						       "/.face",
 						       NULL);
 			char *cfg_file = g_strconcat (g_get_home_dir (),
 						      "/.gnome2/gdm",
@@ -165,6 +236,7 @@ main (int argc, char *argv[])
 							      "reading\nError: %s"),
 							    pixmap,
 							    g_strerror (errno));
+				gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 				gtk_dialog_run (GTK_DIALOG (d));
 				gtk_widget_destroy (d);
 				g_free (cfg_file);
@@ -183,6 +255,7 @@ main (int argc, char *argv[])
 							      "writing\nError: %s"),
 							    photofile,
 							    g_strerror (errno));
+				gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 				gtk_dialog_run (GTK_DIALOG (d));
 				gtk_widget_destroy (d);
 				g_free (cfg_file);
